#include <ps2.h>

/*
 * an arduino sketch to interface with a ps/2 mouse.
 * Also uses serial protocol to talk back to the host
 * and report what it finds.
 */

/*
 * Pin 5 is the mouse data pin, pin 6 is the clock pin
 * Feel free to use whatever pins are convenient.
 */
PS2 mouse(6, 5);
int xrot = 0;
int yrot = 0;

int MBUTTON1 = 2;
int MBUTTON2 = 3;
int MBUTTON3 = 4;
int XDIR = 7;
int XREF = 8;
int YDIR = 9;
int YREF = 10;

int xdir_on = false;
int ydir_on = false;
int xref_on = false;
int yref_on = false;

int xrot_state = 0;
int yrot_state = 0;

int but1_on = false;
int but2_on = false;
int but3_on = false;

/*
 * initialize the mouse. Reset it, and place it into remote
 * mode, so we can get the encoder data on demand.
 */
void mouse_init()
{
  // sets the digital pin as output
  pinMode(MBUTTON1, OUTPUT);
  pinMode(MBUTTON2, OUTPUT);
  pinMode(MBUTTON3, OUTPUT);
  pinMode(XDIR, OUTPUT);
  pinMode(XREF, OUTPUT);  
  pinMode(YDIR, OUTPUT);  
  pinMode(YREF, OUTPUT);
  
  mouse.write(0xff);  // reset
  mouse.read();  // ack byte
  mouse.read();  // blank */
  mouse.read();  // blank */
  mouse.write(0xf0);  // remote mode
  mouse.read();  // ack
  delayMicroseconds(100);
}

void setup()
{
  Serial.begin(9600);
  mouse_init();
}

/*
 * get a reading from the mouse and report it back to the
 * host via the serial line.
 */
void loop()
{
//
//    char mstat;
//  char mx;
//  char my;
//
//  /* get a reading from the mouse */
//  mouse.write(0xeb);  // give me data!
//  mouse.read();      // ignore ack
//  mstat = mouse.read();
//  mx = mouse.read();
//  my = mouse.read();
//
//  /* send the data back up */
//  Serial.print(mstat, BIN);
//  Serial.print("\tX=");
//  Serial.print(mx, DEC);
//  Serial.print("\tY=");
//  Serial.print(my, DEC);
//  Serial.println();
//}

  char mstat;
  char mx;
  char my;

  /* get a reading from the mouse */
  mouse.write(0xeb);  // give me data!
  mouse.read();      // ignore ack
  mstat = mouse.read();
  mx = mouse.read();
  my = mouse.read();

  /* send the data back up */
//  Serial.print(mstat, DEC);
//  Serial.print("\tMSTAT=");
//  Serial.print(mstat, BIN);
//  Serial.print("\tX=");
//  Serial.print(mx, DEC);
//  Serial.print("\tY=");
//  Serial.print(my, DEC);
//  Serial.print("\tXrot=");
//  Serial.print(xrot, BIN);
//  Serial.print(xrot_state, BIN);
//  Serial.println;

  if (mstat & 0b001) {
    if (but1_on == false) {
      digitalWrite(MBUTTON1, LOW);
      but1_on = true;
      Serial.println("button 1 ON");
    }
  }
  else {
    if (but1_on == true) {
      digitalWrite(MBUTTON1, HIGH);    
      but1_on = false;
      Serial.println("button 1 OFF");
    }
  }

  if (mstat & 0b010) {
    if (but2_on == false) {
      digitalWrite(MBUTTON2, LOW);
      but2_on = true;
      Serial.println("button 2 ON");
    }
  }
  else {
    if (but2_on == true) {
      digitalWrite(MBUTTON2, HIGH);    
      but2_on = false;
      Serial.println("button 2 OFF");
    }  
  }
  
  if (mstat & 0b100) {
    if (but3_on == false) {
      digitalWrite(MBUTTON3, LOW);
      but3_on = true;
      Serial.println("button 3 ON");
    }
  }
  else {
    if (but3_on == true) {
      digitalWrite(MBUTTON3, HIGH);    
      but3_on = false;
      Serial.println("button 3 OFF");
    }      
  }

  //Serial.println();

  // Virtual rotary encoder

  // X-axis
  // Mouse moves right steps
  //  0: 1>00<1
  //  1: 0>01<1
  //  3: 0>11<0
  //  2: 1>10<0
  if (mx > 0) {
    if (xrot == 0 && xrot_state == 0) xrot = 1;
    else if (xrot == 1 && xrot_state == 1) xrot = 3;
    else if (xrot == 3 && xrot_state == 3) xrot = 2;
    else if (xrot == 2 && xrot_state == 2) xrot = 0;
  }
  // Mouse moves left steps
  //  2: 1>10<0
  //  3: 0>11<0
  //  1: 0>01<1
  //  0: 1>00<1
  else if (mx < 0) {
    if (xrot == 2 && xrot_state == 2) xrot = 3;
    else if (xrot == 3 && xrot_state == 3) xrot = 1;
    else if (xrot == 1 && xrot_state == 1) xrot = 0;
    else if (xrot == 0 && xrot_state == 0) xrot = 2;
  }


  // Y-axis
  // Mouse moves up steps
  //  0: 1>00<1
  //  1: 0>01<1
  //  3: 0>11<0
  //  2: 1>10<0
  if (my > 0) {
    if (yrot == 0) yrot = 1;
    else if (yrot == 1) yrot = 3;
    else if (yrot == 3) yrot = 2;
    else if (yrot == 2) yrot = 0;
  }
  // Mouse moves down steps
  //  2: 1>10<0
  //  3: 0>11<0
  //  1: 0>01<1
  //  0: 1>00<1
  else if (my < 0) {
    if (yrot == 2) yrot = 3;
    else if (yrot == 3) yrot = 1;
    else if (yrot == 1) yrot = 0;
    else if (yrot == 0) yrot = 2;
  }

  if (xrot & 0b01) {
    if (xdir_on == false) {
      digitalWrite(XDIR, HIGH);
      xdir_on = true;
      xrot_state = xrot_state ^ 0b01; // XOR = toggle bit.
    }
  }
  else {
    if (xdir_on == true) {
      digitalWrite(XDIR, LOW);
      xdir_on = false;
      xrot_state = xrot_state ^ 0b01; //XOR = toggle bit.
    }
  }  

  if (xrot & 0b10) {
    if (xref_on == false) {
      digitalWrite(XREF, HIGH);    
      xref_on = true;
      xrot_state = xrot_state ^ 0b10; // XOR = toggle bit.
    }
  }
  else {
    if (xref_on == true) {
      digitalWrite(XREF, LOW);
      xref_on = false;
      xrot_state = xrot_state ^ 0b10; // XOR = toggle bit.
    }
  }  


  if (yrot & 0b01) {
    if (ydir_on == true) { 
        digitalWrite(YDIR, HIGH);    
        ydir_on = false;
    }
  }
  else {
    if (ydir_on == false) { 
      digitalWrite(YDIR, LOW);
      ydir_on = true;
    }
  }  

  if (yrot & 0b10) {
    if (yref_on == true) { 
      digitalWrite(YREF, HIGH);
      yref_on = false;
    }
  }
  else {
    if (yref_on == false) { 
      digitalWrite(YREF, LOW);        
      yref_on = true;
    }
  }  
  
  
  //delay(20);  /* twiddle */
}

